Crate webtrans_quinn

Crate webtrans_quinn 

Native WebTransport implementation built on top of QUIC using Quinn.

This crate provides a low-level, QUIC WebTransport API for native environments.

The implementation is powered by quinn, and most transport-level behavior (congestion control, flow control, crypto, etc.) is delegated directly to Quinn.

Re-exports

pub use quinn;
pub use rustls;
pub use http;
pub use webtrans_trait as generic;

Modules

crypto
Simple crypto provider utilities for rustls.
tls
TLS helpers for managing certificates and private keys.

Structs

Client
A client for connecting to a WebTransport server.
ClientBuilder
Construct a WebTransport Client using sensible defaults.
ClosedStream
Error indicating the stream was already closed.
DangerousClientBuilder
Builder for dangerous TLS configuration options.
NoCertificateVerification
Certificate verifier that disables all chain and hostname validation.
RecvStream
A stream that can be used to receive bytes. See quinn::RecvStream.
Request
A mostly complete WebTransport handshake awaiting server accept/reject based on URL.
SendStream
A stream that can be used to send bytes. See quinn::SendStream.
Server
A WebTransport server that accepts new sessions.
ServerBuilder
Construct a WebTransport Server using sensible defaults.
Session
An established WebTransport session, acting like a full QUIC connection. See quinn::Connection.
SessionAccept
State machine that accepts and validates incoming streams for a single session.

Enums

ClientError
Error returned when connecting to a WebTransport endpoint.
CongestionControl
Congestion control algorithm to use for the connection.
ConnectError
ReadError
Error when reading from crate::RecvStream, similar to quinn::ReadError.
ReadExactError
Error returned by crate::RecvStream::read_exact, similar to quinn::ReadExactError.
ReadToEndError
Error returned by crate::RecvStream::read_to_end, similar to quinn::ReadToEndError.
ServerError
Error returned when receiving a new WebTransport session.
SessionError
Errors returned by crate::Session, grouped by QUIC or WebTransport origin.
SettingsError
WebTransportError
Error that can occur when reading or writing the WebTransport stream header.
WriteError
Error when writing to crate::SendStream, similar to quinn::WriteError.

Constants

ALPN
The HTTP/3 ALPN token used when negotiating a QUIC connection.
RESET_STREAM_AT_SUPPORTED
Whether the native transport can use the draft-required RESET_STREAM_AT extension.