pub enum ClientError {
Show 14 variants
UnexpectedEnd,
Connection(ConnectionError),
WriteError(WriteError),
ReadError(ReadError),
SettingsError(SettingsError),
HttpError(ConnectError),
QuinnError(ConnectError),
InvalidDnsName(String),
InvalidUrl(String),
DnsTimeout,
HandshakeTimeout,
Io(Arc<Error>),
InvalidCryptoConfiguration,
Rustls(Error),
}Expand description
Error returned when connecting to a WebTransport endpoint.
Variants
UnexpectedEnd
Incoming bytes ended before the handshake exchange completed.
Connection(ConnectionError)
QUIC connection-level failure.
WriteError(WriteError)
Failed to write handshake data.
ReadError(ReadError)
Failed to read handshake data.
SettingsError(SettingsError)
HTTP/3 SETTINGS negotiation failed.
HttpError(ConnectError)
HTTP/3 CONNECT negotiation failed.
QuinnError(ConnectError)
Quinn connect attempt failed before a connection was established.
InvalidDnsName(String)
URL host component could not be converted to a DNS name.
InvalidUrl(String)
URL was invalid for WebTransport usage.
DnsTimeout
DNS resolution exceeded the configured timeout.
HandshakeTimeout
QUIC and HTTP/3 session establishment exceeded the configured timeout.
Io(Arc<Error>)
Local UDP endpoint creation failed.
InvalidCryptoConfiguration
TLS configuration did not provide a QUIC-compatible initial cipher suite.
Rustls(Error)
Rustls-level TLS configuration or handshake error.