pub enum WebTransportError {
Closed(u32, String),
UnknownSession,
ReadError(ReadExactError),
WriteError(WriteError),
}Expand description
Error that can occur when reading or writing the WebTransport stream header.
Variants
Closed(u32, String)
Session was closed with an application code and reason.
UnknownSession
Stream/session header did not match any known session.
ReadError(ReadExactError)
Failed to read stream/session preface data.
WriteError(WriteError)
Failed to write stream/session preface data.