RecvStream

Struct RecvStream 

pub struct RecvStream { /* private fields */ }
A stream that can be used to receive bytes. See quinn::RecvStream.

Implementations

Source

impl RecvStream

pub fn stop(&mut self, code: u32) -> Result<(), ClosedStream>

Tell the peer to stop sending data with the given error code. See quinn::RecvStream::stop. WebTransport uses a u32 because it shares the error space with HTTP/3.

pub async fn read(&mut self, buf: &mut [u8]) -> Result<Option<usize>, ReadError>

Read some data into the buffer and return the amount read. See quinn::RecvStream::read.

pub async fn read_exact(&mut self, buf: &mut [u8]) -> Result<(), ReadExactError>

Fill the entire buffer with data. See quinn::RecvStream::read_exact.

pub async fn read_chunk( &mut self, max_length: usize, ordered: bool, ) -> Result<Option<Chunk>, ReadError>

Read a chunk of data from the stream. See quinn::RecvStream::read_chunk.

pub async fn read_chunks( &mut self, bufs: &mut [Bytes], ) -> Result<Option<usize>, ReadError>

Read chunks of data from the stream. See quinn::RecvStream::read_chunks.

pub async fn read_to_end( &mut self, size_limit: usize, ) -> Result<Vec<u8>, ReadToEndError>

Read until the end of the stream or the limit is hit. See quinn::RecvStream::read_to_end.

pub async fn received_reset(&mut self) -> Result<Option<u32>, SessionError>

Block until the stream has been reset and return the error code. See quinn::RecvStream::received_reset.

Unlike Quinn, this returns SessionError (not ResetError) because 0-RTT is not supported.

pub fn quic_id(&self) -> StreamId

Return the underlying QUIC stream ID.

Warning

WebTransport sessions share the QUIC connection with HTTP/3 and other sessions. The quinn::StreamId::index may not increment by 1 as it does in a standalone quinn connection. The JavaScript WebTransport API therefore does not expose stream IDs.