pub struct Server { /* private fields */ }Expand description
A WebTransport server that accepts new sessions.
Implementations
Sourceimpl Server
impl Server
Sourcepub fn new(endpoint: Endpoint) -> Self
pub fn new(endpoint: Endpoint) -> Self
Manually create a new server with a preconfigured endpoint.
NOTE: The ALPN must be set to crate::ALPN for WebTransport to work.
Sourcepub fn local_addr(&self) -> Result<SocketAddr, Error>
pub fn local_addr(&self) -> Result<SocketAddr, Error>
Return the local address on which the server endpoint is listening.