Native WebTransport implementation built on top of QUIC using Quinn.
This crate provides a low-level, QUIC WebTransport API for native environments.
The implementation is powered by quinn, and most transport-level
behavior (congestion control, flow control, crypto, etc.) is delegated
directly to Quinn.
Re-exports
pub use quinn;pub use rustls;pub use http;pub use webtrans_trait as generic;
Modules
- crypto
- Simple crypto provider utilities for rustls.
- tls
- TLS helpers for managing certificates and private keys.
Structs
- Client
- A client for connecting to a WebTransport server.
- Client
Builder - Construct a WebTransport Client using sensible defaults.
- Closed
Stream - Error indicating the stream was already closed.
- Dangerous
Client Builder - Builder for dangerous TLS configuration options.
- NoCertificate
Verification - Certificate verifier that disables all chain and hostname validation.
- Recv
Stream - A stream that can be used to receive bytes. See
quinn::RecvStream. - Request
- A mostly complete WebTransport handshake awaiting server accept/reject based on URL.
- Send
Stream - A stream that can be used to send bytes. See
quinn::SendStream. - Server
- A WebTransport server that accepts new sessions.
- Server
Builder - Construct a WebTransport Server using sensible defaults.
- Session
- An established WebTransport session, acting like a full QUIC connection. See
quinn::Connection. - Session
Accept - State machine that accepts and validates incoming streams for a single session.
Enums
- Client
Error - Error returned when connecting to a WebTransport endpoint.
- Congestion
Control - Congestion control algorithm to use for the connection.
- Connect
Error - Read
Error - Error when reading from
crate::RecvStream, similar toquinn::ReadError. - Read
Exact Error - Error returned by
crate::RecvStream::read_exact, similar toquinn::ReadExactError. - Read
ToEnd Error - Error returned by
crate::RecvStream::read_to_end, similar toquinn::ReadToEndError. - Server
Error - Error returned when receiving a new WebTransport session.
- Session
Error - Errors returned by
crate::Session, grouped by QUIC or WebTransport origin. - Settings
Error - WebTransport
Error - Error that can occur when reading or writing the WebTransport stream header.
- Write
Error - Error when writing to
crate::SendStream, similar toquinn::WriteError.
Constants
- ALPN
- The HTTP/3 ALPN token used when negotiating a QUIC connection.
- RESET_
STREAM_ AT_ SUPPORTED - Whether the native transport can use the draft-required RESET_STREAM_AT extension.